@@ -1,6 +1,5 @@
 import csv
 from .quality_indicator import gd, epsilon_add
-from matplotlib import pylab, rc, axis
 
 
 class Results:
@@ -8,24 +7,6 @@ class Results:
 
     def __init__(self, problem):
         self.problem = problem
-        self.use_latex = True
-        self.label_size = 16
-        self.tick_size = 14
-        self.title_size = 18
-        self.legend = True
-        self.width = 8
-        self.hight = 5
-
-        if self.use_latex:
-            # allows using Latex
-            rc('font', **{'family': ' DejaVu Sans', 'sans-serif': ['Helvetica']})
-            rc('text', usetex=True)
-
-        # sets the legend font size
-        params = {'legend.fontsize': 14,
-                  'legend.handlelength': 2}
-
-        pylab.rcParams.update(params)
 
     def parameter_names(self):
         parameter_names = []
@@ -51,13 +32,15 @@ class Results:
         except ValueError as e:
             raise ValueError('There is not a parameter with given name: {}'.format(name))
 
+    # ToDo: Replace exception raise by warning?
     def goal_index(self, name):
-        try:
-            return next((index for (index, d) in enumerate(self.problem.costs) if d["name"] == name), None)
-            # return list(self.problem.costs).index(name)
-        except ValueError as e:
+        index = next((index for (index, d) in enumerate(self.problem.costs) if d["name"] == name), None)
+        if index is not None:
+            return index
+        else:
             raise ValueError('There is not a cost function with given name: {}'.format(name))
 
+
     def parameters(self):
         out = []
         for individuals in self.problem.populations().values():
@@ -83,19 +66,26 @@ class Results:
         else:
             return out
 
+    # TODO : This method returns all the populations instead of the population in the given index
     def population(self, population_id=-1):
         # Returns population with given index
-        #if population_id >= len(self.problem.populations):
+        #   if population_id >= len(self.problem.populations):
         #    raise ValueError('Index of population exceeds the number of populations.')
         if population_id == -1:
+            # TODO : last_population returns all the populations instead of last index population
             individuals = self.problem.last_population()
         else:
+            # TODO : population() must be modified to return the given index population
             individuals = self.problem.population(population_id)
         return individuals
 
+    # I assume this method must return the goal on the given index not to return all the population or the name must
+    # change
     def goal_on_index(self, name=None, population_id=-1):
-        """ Returns a list of lists. The first list contains indexes of individuals in population,
-            the other lists contains values of the goal function(s).
+        """
+        Returns a list of lists. The first list contains indexes of individuals in population,
+        the other lists contains values of the goal function(s).
+
         """
         individuals = self.population(population_id)
         n = range(len(individuals))
@@ -109,11 +99,14 @@ class Results:
         else:
             table.append([])
             index = self.goal_index(name)
+
             for i in n:
                 table[1].append(individuals[i].costs[index])
 
         return table
 
+    # I assume this method must return the goal on the given index not to return all the population or the name must
+    # change
     def parameter_on_index(self, name=None, population_id=-1):
         """
         Returns a list of lists. The first list contains indexes of individuals in population,
@@ -221,7 +214,26 @@ class Results:
                         out.append(j)
 
                     writer.writerows([out])
-                population_id += 1
+                    # population_id += 1
+                    # increasing population_id does not work properly here, it must be written as below
+                    population_id += 1
+
+    def pareto_individuals(self, population_id=None):
+        """
+
+        :return: pareto front
+        """
+        if population_id is not None:
+            individuals = self.population(population_id)
+        else:
+            individuals = self.population(-1)
+
+        pareto_individuals = []
+        for individual in individuals:
+            if individual.features['front_number'] == 1:
+                pareto_individuals.append(individual)
+
+        return pareto_individuals
 
     def pareto_front(self, population_id=None):
         """
@@ -231,7 +243,7 @@ class Results:
         if population_id is not None:
             individuals = self.population(population_id)
         else:
-            individuals = self.problem.individuals
+            individuals = self.population(-1)
 
         n = self.goal_number()
         pareto_front = []
@@ -259,25 +271,29 @@ class Results:
         :return:
         """
 
+
         # get the index of the required parameter
         index = 0  # default - one parameter
+        min_l = []
         if name:
-            for i in range(len(self.problem.costs)):
-                cost = self.problem.costs[i]
-                if cost['name'] == name:
-                    index = i
-        # if len(self.problem.data_store.individuals) is not 0:
-        #    min_l = [min(self.problem.data_store.individuals, key=lambda x: x.costs[index])]
-        # else:
-        #if self.problem.archive:
-        #    min_l = [min(self.problem.archive, key=lambda x: x.costs[index])]
-        #else:
-        if len(self.problem.individuals) > 0:
-            min_l = [min(self.problem.individuals, key=lambda x: x.costs[index])]
+            index = self.goal_index(name)
+
+        criteria = None
+        if 'criteria' in self.problem.costs[index]:
+            criteria = self.problem.costs[index]['criteria']
+
+        if criteria == 'minimize' or criteria is None:
+            if len(self.problem.individuals) > 0:
+                min_l = [min(self.problem.individuals, key=lambda x: x.costs[index])]
+        else:
+            if len(self.problem.individuals) > 0:
+                min_l = [max(self.problem.individuals, key=lambda x: x.costs[index])]
+
         # for population in self.problem.populations:
         opt = min(min_l, key=lambda x: x.costs[index])
         return opt
 
+
     # TODO: same function - David, could you check it and write test?
     def pareto_values(self, archive=None):
         """
@@ -310,6 +326,7 @@ class Results:
         :return:
         """
 
+        result = 0
         computed = self.pareto_values()  # [[], [], .. ]
 
         if type == 'epsilon':
@@ -319,123 +336,9 @@ class Results:
 
         return result
 
-    def set_figure(self):
-        # We change the fontsize of minor ticks label
-        fig, ax = pylab.subplots(figsize=(self.width, self.hight))
-        ax.tick_params(axis='both', which='major', labelsize=self.tick_size)
-        ax.tick_params(axis='both', which='minor', labelsize=self.tick_size)
-        ax.grid()
-        # ax.grid(color='grey', linestyle='-', linewidth=0.25, alpha=0.5)
-        # removing top and right borders
-        # ax.spines['top'].set_visible(False)
-        # ax.spines['right'].set_visible(False)  # adds major gridlines
-        return [fig, ax]
 
-    def get_objectives_plot(self):
-
-        fig, ax = self.set_figure()
-        all_individuals = self.costs()
-        for point in all_individuals:
-            ax.plot(point[0], point[1], '.')
-
-        pareto = self.pareto_front()
-        ax.plot(pareto[0], pareto[1], 'ro', label='Last Pareto front')
-
-        ax.set_xlabel(r'$F_1$', fontsize=self.label_size)
-        ax.set_ylabel(r'$F_2$', fontsize=self.label_size)
-        ax.legend()
-        return [fig, ax]
-
-    def objectives_plot(self):
-        fig, ax = self.get_objectives_plot()
-        fig.show()
-
-    def get_pareto_plot(self, cost_x=0, cost_y=1):
-        """
-        A simple 2d - pareto plot from the variable 1 and 2 by default.
-        :return:
-        """
-        fig, ax = self.set_figure()
-        pvalues = self.pareto_values()
-
-        x = [i[cost_x] for i in pvalues]
-        y = [i[cost_y] for i in pvalues]
-
-        ax.scatter(x, y, alpha=0.80)
-
-        # adds a title and axes labels
-        x_name = self.problem.costs[cost_x]['name']
-        y_name = self.problem.costs[cost_y]['name']
-
-        ax.set_title('Pareto values', fontsize=self.title_size)
-        ax.set_xlabel('$' + x_name + '$', fontsize=self.label_size)
-        ax.set_ylabel('$' + y_name + '$', fontsize=self.label_size)
-        return [fig, ax]
-
-
-    def pareto_plot(self, cost_x=0, cost_y=1):
-        fig, ax = self.get_pareto_plot(cost_x, cost_y)
-        fig.show()
-
-    def get_goal_on_index(self, name, population_id=-1):
-        fig, ax = self.set_figure()
-        table = self.goal_on_index(name=name, population_id=population_id)
-        ax.plot(table[0], table[1])
-
-        ax.set_xlabel(r'$k$', fontsize=self.label_size)
-        ax.set_ylabel(r'$' + name + '$', fontsize=self.label_size)
-        return [fig, ax]
-
-    def goal_on_index_plot(self, name, population_id=-1):
-        fig, ax = self.get_goal_on_index(name, population_id)
-        fig.show()
-
-    def get_parameter_on_index(self, name, population_id=-1):
-        fig, ax = self.set_figure()
-        table = self.parameter_on_index(name=name, population_id=population_id)
-        ax.plot(table[0], table[1])
-
-        ax.set_xlabel(r'$k$', fontsize=self.label_size)
-        ax.set_ylabel(r'$' + name + '$', fontsize=self.label_size)
-        return [fig, ax]
-
-    def parameter_on_index_plot(self, name, population_id=-1):
-        fig, ax = self.get_parameter_on_index(name, population_id)
-        fig.show()
-
-    def get_goal_on_parameter(self, parameter_name, goal_name, population_id=-1, sorted=False):
-        fig, ax = self.set_figure()
-        table = self.goal_on_parameter(parameter_name, goal_name, population_id, sorted)
-        ax.plot(table[0], table[1], 'o')
-
-        ax.set_xlabel(r'$' + parameter_name + '$', fontsize=self.label_size)
-        ax.set_ylabel(r'$' + goal_name + '$', fontsize=self.label_size)
-        return [fig, ax]
-
-    def goal_on_parameter_plot(self, parameter_name, goal_name, population_id=-1, sorted=False):
-        fig, ax = self.get_goal_on_parameter(parameter_name, goal_name, population_id, sorted)
-        fig.show()
-
-    def get_parameter_on_parameter(self, parameter_1, parameter_2, population_id=-1, sorted=False):
-        fig, ax = self.set_figure()
-        table = self.parameter_on_parameter(parameter_1, parameter_2, population_id, sorted)
-        ax.plot(table[0], table[1], 'o')
-
-        ax.set_xlabel(r'$' + parameter_1 + '$', fontsize=self.label_size)
-        ax.set_ylabel(r'$' + parameter_2 + '$', fontsize=self.label_size)
-        return [fig, ax]
-
-    def parameter_on_parameter_plot(self, parameter_1, parameter_2, population_id=-1, sorted=False):
-        fig, ax = self.get_parameter_on_parameter(parameter_1, parameter_2, population_id, sorted)
-        fig.show()
-
-    def goal_histogram_plot(self, name, population_id=-1):
-        fig, ax = self.set_figure()
-        table = self.goal_on_index(name=name, population_id=population_id)
-        n, bins, patches = ax.hist(x=table[1], bins='auto', color='#0504aa',
-                                    alpha=0.7, rwidth=0.85)
-        fig.show()
-
-    def get_mean_confidence_interval(self, name):
-        index = self.goal_index(name)
-        return 
+    def get_population_ids(self):
+        ids = set()
+        for individual in self.problem.individuals:
+            ids.add(individual.population_id)
+        return ids
