@@ -1,6 +1,6 @@
 import unittest
 from ..operators import CustomGenerator, RandomGenerator, FullFactorGenerator, FullFactorLevelsGenerator, PlackettBurmanGenerator, \
-    BoxBehnkenGenerator, LHSGenerator, GSDGenerator, HaltonGenerator
+    BoxBehnkenGenerator, LHSGenerator, GSDGenerator, HaltonGenerator, UniformGenerator
 
 
 class TestDOE(unittest.TestCase):
@@ -37,6 +37,20 @@ class TestDOE(unittest.TestCase):
         self.assertEqual(individuals[1].vector[1], 3)
         self.assertEqual(individuals[1].vector[2], 8)
 
+    def test_uniform_generator(self):
+        gen = UniformGenerator(parameters=self.parameters)
+        gen.init(4)
+        individuals = gen.generate()
+
+        # values
+        self.assertEqual(len(individuals), 4**len(self.parameters))
+        self.assertEqual(individuals[0].vector[2], 6.0)
+        self.assertEqual(individuals[1].vector[0], -2.5)
+        self.assertEqual(individuals[12].vector[1], 3.4)
+        self.assertEqual(individuals[28].vector[2], 6.0)
+        self.assertEqual(individuals[35].vector[0], 2.5)
+        self.assertEqual(individuals[63].vector[2], 10.0)
+
     # Factorial Designs
     def test_full_fact_generator(self):
         # General Full-Factorial
